<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 04/04/2009 *
 * ===============                              Last change: 04/04/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : wrapper-functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Wrapper functions                                *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Wrapper-Funktionen                               *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Read a given file
function readFromFile ($FQFN, $sqlPrepare = false) {
	// Sanity-check if file is there (should be there, but just to make it sure)
	if (!isFileReadable($FQFN)) {
		// This should not happen
		debug_report_bug(__FUNCTION__.': File ' . basename($FQFN) . ' is not readable!');
	} // END - if

	// Load the file
	if (function_exists('file_get_contents')) {
		// Use new function
		$content = file_get_contents($FQFN);
	} else {
		// Fall-back to implode-file chain
		$content = implode('', file($FQFN));
	}

	// Prepare SQL queries?
	if ($sqlPrepare === true) {
		// Remove some unwanted chars
		$content = str_replace("\r", '', $content);
		$content = str_replace("\n\n", "\n", $content);
	} // END - if

	// Return the content
	return $content;
}

// Writes content to a file
function writeToFile ($FQFN, $content, $aquireLock = false) {
	// Is the file writeable?
	if ((isFileReadable($FQFN)) && (!is_writeable($FQFN)) && (!changeMode($FQFN, 0644))) {
		// Not writeable!
		logDebugMessage(__FUNCTION__, __LINE__, sprintf("File %s not writeable.", basename($FQFN)));

		// Failed! :(
		return false;
	} // END - if

	// By default all is failed...
	$return = false;

	// Is the function there?
	if (function_exists('file_put_contents')) {
		// With lock?
		if ($aquireLock === true) {
			// Write it directly with lock
			$return = file_put_contents($FQFN, $content, LOCK_EX);
		} else {
			// Write it directly
			$return = file_put_contents($FQFN, $content);
		}
	} else {
		// Write it with fopen
		$fp = fopen($FQFN, 'w') or app_die(__FUNCTION__, __LINE__, "Cannot write file ".basename($FQFN).'!');

		// Aquire lock
		if ($aquireLock === true) flock($fp, LOCK_EX);

		// Write content
		fwrite($fp, $content);

		// Close stream
		fclose($fp);
	}

	// Mark it as readable
	$GLOBALS['file_readable'][$FQFN] = true;

	// Return status
	return changeMode($FQFN, 0644);
}

// Clears the output buffer. This function does *NOT* backup sent content.
function clearOutputBuffer () {
	// Trigger an error on failure
	if (!ob_end_clean()) {
		// Failed!
		debug_report_bug(__FUNCTION__.': Failed to clean output buffer.');
	} // END - if
}

// Encode strings
// @TODO Implement $compress
function encodeString ($str, $compress = true) {
	$str = urlencode(base64_encode(compileUriCode($str)));
	return $str;
}

// Decode strings encoded with encodeString()
// @TODO Implement $decompress
function decodeString ($str, $decompress = true) {
	$str = compileUriCode(base64_decode(urldecode(compileUriCode($str))));
	return $str;
}

// Smartly adds slashes
function smartAddSlashes ($unquoted) {
	// Do we have cache?
	if (!isset($GLOBALS['smart_addslashes'][$unquoted])) {
		// Remove slashe
		$unquoted = str_replace("\\", '', $unquoted);

		// Put it in cache and add slashes
		$GLOBALS['smart_addslashes'][$unquoted] = addslashes($unquoted);
	} // END - if

	// Return result
	return $GLOBALS['smart_addslashes'][$unquoted];
}

// Decode entities in a nicer way
function decodeEntities ($str) {
	// Decode the entities to UTF-8 now
	$decodedString = html_entity_decode($str, ENT_NOQUOTES, 'UTF-8');

	// Return decoded string
	return $decodedString;
}

// Merges an array together but only if both are arrays
function merge_array ($array1, $array2) {
	// Are both an array?
	if ((!is_array($array1)) && (!is_array($array2))) {
		// Both are not arrays
		debug_report_bug(__FUNCTION__ . ': No arrays provided!');
	} elseif (!is_array($array1)) {
		// Left one is not an array
		debug_report_bug(sprintf("[%s:%s] array1 is not an array. array != %s", __FUNCTION__, __LINE__, gettype($array1)));
	} elseif (!is_array($array2)) {
		// Right one is not an array
		debug_report_bug(sprintf("[%s:%s] array2 is not an array. array != %s", __FUNCTION__, __LINE__, gettype($array2)));
	}

	// Merge all together
	return array_merge($array1, $array2);
}

// Check if given FQFN is a readable file
function isFileReadable ($FQFN) {
	// Do we have cache?
	if (!isset($GLOBALS['file_readable'][$FQFN])) {
		// Check all...
		$GLOBALS['file_readable'][$FQFN] = ((file_exists($FQFN)) && (is_file($FQFN)) && (is_readable($FQFN)));
	} // END - if

	// Return result
	return $GLOBALS['file_readable'][$FQFN];
}

// Checks wether the given FQFN is a directory and not ., .. or .svn
function isDirectory ($FQFN) {
	// Do we have cache?
	if (!isset($GLOBALS['is_directory'][$FQFN])) {
		// Generate baseName
		$baseName = basename($FQFN);

		// Check it
		$GLOBALS['is_directory'][$FQFN] = ((is_dir($FQFN)) && ($baseName != '.') && ($baseName != '..') && ($baseName != '.svn'));
	} // END - if

	// Return the result
	return $GLOBALS['is_directory'][$FQFN];
}

// "Getter" for remote IP number
function detectRemoteAddr () {
	// Get remote ip from environment
	$remoteAddr = determineRealRemoteAddress();

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteAddr = getAnonymousRemoteAddress($remoteAddr);
	} // END - if

	// Return it
	return $remoteAddr;
}

// "Getter" for remote hostname
function detectRemoteHostname () {
	// Get remote ip from environment
	$remoteHost = getenv('REMOTE_HOST');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteHost = getAnonymousRemoteHost($remoteHost);
	} // END - if

	// Return it
	return $remoteHost;
}

// "Getter" for user agent
function detectUserAgent () {
	// Get remote ip from environment
	$userAgent = getenv('HTTP_USER_AGENT');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$userAgent = getAnonymousUserAgent($userAgent);
	} // END - if

	// Return it
	return $userAgent;
}

// "Getter" for referer
function detectReferer () {
	// Get remote ip from environment
	$referer = getenv('HTTP_REFERER');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$referer = getAnonymousReferer($referer);
	} // END - if

	// Return it
	return $referer;
}

// Check wether we are installing
function isInstalling () {
	// Determine wether we are installing
	if (!isset($GLOBALS['mxchange_installing'])) {
		// Check URL (css.php/js.php need this)
		$GLOBALS['mxchange_installing'] = isGetRequestElementSet('installing');
	} // END - if

	// Return result
	return $GLOBALS['mxchange_installing'];
}

// Check wether this script is installed
function isInstalled () {
	// Do we have cache?
	if (!isset($GLOBALS['is_installed'])) {
		// Determine wether this script is installed
		$GLOBALS['is_installed'] = (
		(
			// First is config
			(
				(
					isConfigEntrySet('MXCHANGE_INSTALLED')
				) && (
					getConfig('MXCHANGE_INSTALLED') == 'Y'
				)
			)
		) || (
			// New config file found and loaded
			isIncludeReadable(getConfig('CACHE_PATH') . 'config-local.php')
		) || (
			(
				// New config file found, but not yet read
				isIncludeReadable(getConfig('CACHE_PATH') . 'config-local.php')
			) && (
				(
					// Only new config file is found
					!isIncludeReadable('inc/config.php')
				) || (
					// Is installation mode
					!isInstalling()
				)
			)
		));
	} // END - if

	// Then use the cache
	return $GLOBALS['is_installed'];
}

// Check wether an admin is registered
function isAdminRegistered () {
	return ((isConfigEntrySet('ADMIN_REGISTERED')) && (getConfig('ADMIN_REGISTERED') == 'Y'));
}

// Checks wether the reset mode is active
function isResetModeEnabled () {
	// Now simply check it
	return ((isset($GLOBALS['reset_enabled'])) && ($GLOBALS['reset_enabled'] === true));
}

// Checks wether the debug mode is enabled
function isDebugModeEnabled () {
	// Simply check it
	return ((isConfigEntrySet('DEBUG_MODE')) && (getConfig('DEBUG_MODE') == 'Y'));
}

// Checks wether we shall debug regular expressions
function isDebugRegExpressionEnabled () {
	// Simply check it
	return ((isConfigEntrySet('DEBUG_REGEX')) && (getConfig('DEBUG_REGEX') == 'Y'));
}

// Checks wether the cache instance is valid
function isCacheInstanceValid () {
	return ((isset($GLOBALS['cache_instance'])) && (is_object($GLOBALS['cache_instance'])));
}

// Copies a file from source to destination and verifies if that goes fine.
// This function should wrap the copy() command and make a nicer debug backtrace
// even if there is no xdebug extension installed.
function copyFileVerified ($source, $dest, $chmod = '') {
	// Failed is the default
	$status = false;

	// Is the source file there?
	if (!isFileReadable($source)) {
		// Then abort here
		debug_report_bug('Cannot read from source file ' . basename($source) . '.');
	} // END - if

	// Is the target directory there?
	if (!isDirectory(dirname($dest))) {
		// Then abort here
		debug_report_bug('Cannot find directory ' . str_replace(getConfig('PATH'), '', dirname($dest)) . '.');
	} // END - if

	// Now try to copy it
	if (!copy($source, $dest)) {
		// Something went wrong
		debug_report_bug('copy() has failed to copy the file.');
	} else {
		// Reset cache
		$GLOBALS['file_readable'][$dest] = true;
	}

	// If there are chmod rights set, apply them
	if (!empty($chmod)) {
		// Try to apply them
		$status = changeMode($dest, $chmod);
	} else {
		// All fine
		$status = true;
	}

	// All fine
	return $status;
}

// Wrapper function for header()
// Send a header but checks before if we can do so
function sendHeader ($header) {
	// Is the header already sent?
	if (headers_sent()) {
		// Then abort here
		debug_report_bug('Headers already sent!');
	} // END - if

	// Send the header
	header(trim($header));
}

// Wrapper function for chmod()
// @TODO Do some more sanity check here
function changeMode ($FQFN, $mode) {
	// Is the file/directory there?
	if ((!isFileReadable($FQFN)) && (!isDirectory($FQFN))) {
		// Neither, so abort here
		debug_report_bug('Cannot chmod() on ' . basename($FQFN) . '.');
	} // END - if

	// Try to set them
	chmod($FQFN, $mode);
}

// Wrapper for unlink()
function removeFile ($FQFN) {
	// Is the file there?
	if (isFileReadable($FQFN)) {
		// Reset cache first
		$GLOBALS['file_readable'][$FQFN] = false;

		// Yes, so remove it
		return unlink($FQFN);
	} // END - if

	// All fine if no file was removed. If we change this to 'false' or rewrite
	// above if() block it would be to restrictive.
	return true;
}

// Wrapper for $_POST['sel']
function countPostSelection () {
	return countSelection(postRequestElement('sel'));
}

// Checks wether the config-local.php is loaded
function isConfigLocalLoaded () {
	return ((isset($GLOBALS['config_local_loaded'])) && ($GLOBALS['config_local_loaded'] === true));
}

// Checks wether a nickname or userid was entered and caches the result
function isNicknameUsed ($userid) {
	// Default is false
	$isUsed = false;

	// Is the cache there
	if (isset($GLOBALS['cache_probe_nicknames'][$userid])) {
		// Then use it
		$isUsed = $GLOBALS['cache_probe_nicknames'][$userid];
	} else {
		// Determine it
		$isUsed = ((isExtensionActive('nickname')) && (('' . round($userid) . '') != $userid));

		// And write it to the cache
		$GLOBALS['cache_probe_nicknames'][$userid] = $isUsed;
	}

	// Return the result
	return $isUsed;
}

// Getter for 'what' value
function getWhat () {
	// Default is null
	$what = null;

	// Is the value set?
	if (isWhatSet(true)) {
		// Then use it
		$what = $GLOBALS['what'];
	} // END - if

	// Return it
	return $what;
}

// Setter for 'what' value
function setWhat ($newWhat) {
	$GLOBALS['what'] = SQL_ESCAPE($newWhat);
}

// Setter for 'what' from configuration
function setWhatFromConfig ($configEntry) {
	// Get 'what' from config
	$what = getConfig($configEntry);

	// Set it
	setWhat($what);
}

// Checks wether what is set and optionally aborts on miss
function isWhatSet ($abortOnMiss =  false) {
	// Check for it
	$isset = (isset($GLOBALS['what']));

	// Should we abort here?
	if (($abortOnMiss === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug('what is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'action' value
function getAction () {
	// Default is null
	$action = null;

	// Is the value set?
	if (isActionSet(true)) {
		// Then use it
		$action = $GLOBALS['action'];
	} // END - if

	// Return it
	return $action;
}

// Setter for 'action' value
function setAction ($newAction) {
	$GLOBALS['action'] = SQL_ESCAPE($newAction);
}

// Checks wether action is set and optionally aborts on miss
function isActionSet ($abortOnMiss =  false) {
	// Check for it
	$isset = (isset($GLOBALS['action']));

	// Should we abort here?
	if (($abortOnMiss === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug('action is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'module' value
function getModule () {
	// Default is null
	$module = null;

	// Is the value set?
	if (isModuleSet(true)) {
		// Then use it
		$module = $GLOBALS['module'];
	} // END - if

	// Return it
	return $module;
}

// Setter for 'module' value
function setModule ($newModule) {
	// Secure it and make all modules lower-case
	$GLOBALS['module'] = SQL_ESCAPE(strtolower($newModule));
}

// Checks wether module is set and optionally aborts on miss
function isModuleSet ($abortOnMiss =  false) {
	// Check for it
	$isset = (!empty($GLOBALS['module']));

	// Should we abort here?
	if (($abortOnMiss === true) && ($isset === false)) {
		// Output backtrace
		print '<pre>';
		debug_print_backtrace();
		die('</pre');
		debug_report_bug('module is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'output_mode' value
function getOutputMode () {
	// Default is null
	$output_mode = null;

	// Is the value set?
	if (isOutputModeSet(true)) {
		// Then use it
		$output_mode = $GLOBALS['output_mode'];
	} // END - if

	// Return it
	return $output_mode;
}

// Setter for 'output_mode' value
function setOutputMode ($newOutputMode) {
	$GLOBALS['output_mode'] = SQL_ESCAPE($newOutputMode);
}

// Checks wether output_mode is set and optionally aborts on miss
function isOutputModeSet ($abortOnMiss =  false) {
	// Check for it
	$isset = (isset($GLOBALS['output_mode']));

	// Should we abort here?
	if (($abortOnMiss === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug('output_mode is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Enables block-mode
function enableBlockMode ($enabled = true) {
	$GLOBALS['block_mode'] = $enabled;
}

// Checks wether block-mode is enabled
function isBlockModeEnabled () {
	// Abort if not set
	if (!isset($GLOBALS['block_mode'])) {
		// Needs to be fixed
		debug_report_bug(__FUNCTION__ . ': block_mode is not set.');
	} // END - if

	// Return it
	return $GLOBALS['block_mode'];
}

// Wrapper function for addPointsThroughReferalSystem()
function addPointsDirectly ($subject, $userid, $points) {
	// Reset level here
	unset($GLOBALS['ref_level']);

	// Call more complicated method (due to more parameters)
	return addPointsThroughReferalSystem($subject, $userid, $points, false, 0, false, 'direct');
}

// Wrapper function to redirect from member-only modules to index
function redirectToIndexMemberOnlyModule () {
	// Do the redirect here
	redirectToUrl('modules.php?module=index&amp;code=' . getCode('MODULE_MEM_ONLY') . '&amp;mod=' . getModule());
}

// Wrapper function for checking if extension is installed and newer or same version
function isExtensionInstalledAndNewer ($ext_name, $version) {
	// Return it
	return ((isExtensionInstalled($ext_name)) && (getExtensionVersion($ext_name) >= $version));
}

// Wrapper function for checking if extension is installed and older than given version
function isExtensionInstalledAndOlder ($ext_name, $version) {
	// Return it
	return ((isExtensionInstalled($ext_name)) && (isExtensionOlder($ext_name, $version)));
}

// Set username
function setUsername ($userName) {
	$GLOBALS['username'] = (string) $userName;
}

// Get username
function getUsername () {
	return $GLOBALS['username'];
}

// Wrapper function for installation phase
function isInstallationPhase () {
	// Do we have cache?
	if (!isset($GLOBALS['installation_phase'])) {
		// Determine it
		$GLOBALS['installation_phase'] = ((!isInstalled()) || (isInstalling()));
	} // END - if

	// Return result
	return $GLOBALS['installation_phase'];
}

// Checks wether the extension demo is actuve and the admin login is demo (password needs to be demo, too!)
function isDemoModeActive () {
	return ((isExtensionActive('demo')) && (getSession('admin_login') == 'demo'));
}

// Wrapper function to redirect to de-refered URL
function redirectToDereferedUrl ($URL) {
	// De-refer the URL
	$URL = generateDerefererUrl($URL);

	// Redirect to to
	redirectToUrl($URL);
}

// Getter for PHP caching value
function getPhpCaching () {
	return $GLOBALS['php_caching'];
}

// Checks wether the admin hash is set
function isAdminHashSet ($admin) {
	return isset($GLOBALS['cache_array']['admin']['password'][$admin]);
}

// Setter for admin hash
function setAdminHash ($admin, $hash) {
	$GLOBALS['cache_array']['admin']['password'][$admin] = $hash;
}

// [EOF]
?>
